module scomp(clock, reset, memory_address, memory_data_out, register_A, pc, state);
input clock, reset;
output [15:0] memory_data_out;
output [15:0] register_A;
output [7:0] pc;
output [7:0] memory_address;
output [7:0] state;
reg [7:0] state;
reg write_enable;
reg [15:0] register_A;
reg [7:0] pc, memory_address;
parameter
	reset_pc = 8'h80,
	fetch = 8'h81,
	store1 = 8'h82;
// instantiate memory (256 16-bit words)
mem mem_component(
	.data (register_A),
	.address (memory_address),
	.we (write_enable),
	.q (memory_data_out),
	.inclock (clock)
	);
// finite state machine
	always @(negedge clock, posedge reset)
	begin
	if (reset) state<=reset_pc;
	else
	case (state)
	reset_pc :
		begin
		pc <= 8'h00;
		write_enable <= 1'b0;
		memory_address <= 8'h00;
		register_A <= 16'h0000;
		state <= fetch;
		end
	// latch instruction address, increment program counter
	fetch :
		begin
		pc <= pc + 8'h01;
		// fetch instruction (on posedge), setup data address
		memory_address <= memory_data_out[7:0];
		state <= memory_data_out[15:8];
		end
	// execute instruction, setup instruction address
	// Add opcode
	8'h00 :
		begin
		register_A <= register_A + memory_data_out;
		memory_address <= pc;
		state <= fetch;
		end
	// store
	8'h01 :
		begin
		write_enable <= 1'b1;
		state <= store1;
		end
	store1 :
		begin
		memory_address <= pc;
		write_enable <= 1'b0;
		state <= fetch;
		end
	// load
	8'h02 :
		begin
		register_A <= memory_data_out;
		memory_address <= pc;
		state <= fetch;
		end
	// jump
	8'h03 :
		begin
		pc <= memory_address;
		state <= fetch;
		end
	
	// unrecognized state/opcode
	default :
		begin
		memory_address <= pc;
		state <= fetch;
		end
	endcase
	end
endmodule